Method: Pkg::Retrieve.default_wget

Defined in:
lib/packaging/retrieve.rb

.default_wget(local_target, url, additional_options = {}) ⇒ Object

NOTE: When supplying additional options, if you want your value to be quoted (e.g. –reject=‘index*’), you must include the quotes as part of your string (e.g. => “‘index*’”).



40
41
42
43
44
# File 'lib/packaging/retrieve.rb', line 40

def default_wget(local_target, url, additional_options = {})
  wget_command = default_wget_command(local_target, url, additional_options)
  puts "Executing #{wget_command} . . ."
  %x(#{wget_command})
end