Module: Pkg::Platforms
- Included in:
- Paths
- Defined in:
- lib/packaging/platforms.rb
Constant Summary collapse
- DEBIAN_SOURCE_FORMATS =
['debian.tar.gz', 'orig.tar.gz', 'dsc', 'changes']
- PLATFORM_INFO =
Each element in this hash
{ 'aix' => { '6.1' => { architectures: ['power'], source_architecture: 'SRPMS', package_format: 'rpm', source_package_formats: ['src.rpm'], repo: false, }, '7.1' => { architectures: ['power'], source_architecture: 'SRPMS', package_format: 'rpm', source_package_formats: ['src.rpm'], repo: false, }, }, 'cisco-wrlinux' => { '5' => { architectures: ['x86_64'], source_architecture: 'SRPMS', package_format: 'rpm', source_package_formats: ['src.rpm'], signature_format: 'v4', repo: true, }, '7' => { architectures: ['x86_64'], source_architecture: 'SRPMS', package_format: 'rpm', source_package_formats: ['src.rpm'], signature_format: 'v4', repo: true, }, }, 'cumulus' => { '2.2' => { codename: 'cumulus', architectures: ['amd64'], source_architecture: 'source', package_format: 'deb', source_package_formats: DEBIAN_SOURCE_FORMATS, repo: true, }, }, 'debian' => { '7' => { codename: 'wheezy', architectures: ['amd64', 'i386'], source_architecture: 'source', package_format: 'deb', source_package_formats: DEBIAN_SOURCE_FORMATS, repo: true, }, '8' => { codename: 'jessie', architectures: ['amd64', 'i386', 'powerpc'], source_architecture: 'source', package_format: 'deb', source_package_formats: DEBIAN_SOURCE_FORMATS, repo: true, }, '9' => { codename: 'stretch', architectures: ['amd64', 'i386'], source_architecture: 'source', package_format: 'deb', source_package_formats: DEBIAN_SOURCE_FORMATS, repo: true, }, '10' => { codename: 'buster', architectures: ['amd64', 'i386'], source_architecture: 'source', package_format: 'deb', source_package_formats: DEBIAN_SOURCE_FORMATS, repo: true, }, }, 'el' => { '5' => { architectures: ['x86_64', 'i386'], source_architecture: 'SRPMS', package_format: 'rpm', source_package_formats: ['src.rpm'], signature_format: 'v3', repo: true, }, '6' => { architectures: ['x86_64', 'i386', 's390x'], source_architecture: 'SRPMS', package_format: 'rpm', source_package_formats: ['src.rpm'], signature_format: 'v4', repo: true, }, '7' => { architectures: ['x86_64', 's390x', 'ppc64le', 'aarch64'], source_architecture: 'SRPMS', package_format: 'rpm', source_package_formats: ['src.rpm'], signature_format: 'v4', repo: true, }, }, 'eos' => { '4' => { architectures: ['i386'], package_format: 'swix', repo: false, }, }, 'fedora' => { 'f25' => { architectures: ['x86_64', 'i386'], source_architecture: 'SRPMS', package_format: 'rpm', source_package_formats: ['src.rpm'], signature_format: 'v4', repo: true, }, 'f26' => { architectures: ['x86_64'], source_architecture: 'SRPMS', package_format: 'rpm', source_package_formats: ['src.rpm'], signature_format: 'v4', repo: true, }, 'f27' => { architectures: ['x86_64'], source_architecture: 'SRPMS', package_format: 'rpm', source_package_formats: ['src.rpm'], signature_format: 'v4', repo: true, }, 'f28' => { architectures: ['x86_64'], source_architecture: 'SRPMS', package_format: 'rpm', source_package_formats: ['src.rpm'], signature_format: 'v4', repo: true, }, '25' => { architectures: ['x86_64', 'i386'], source_architecture: 'SRPMS', package_format: 'rpm', source_package_formats: ['src.rpm'], signature_format: 'v4', repo: true, }, '26' => { architectures: ['x86_64'], source_architecture: 'SRPMS', package_format: 'rpm', source_package_formats: ['src.rpm'], signature_format: 'v4', repo: true, }, '27' => { architectures: ['x86_64'], source_architecture: 'SRPMS', package_format: 'rpm', source_package_formats: ['src.rpm'], signature_format: 'v4', repo: true, }, '28' => { architectures: ['x86_64'], source_architecture: 'SRPMS', package_format: 'rpm', source_package_formats: ['src.rpm'], signature_format: 'v4', repo: true, }, '29' => { architectures: ['x86_64'], source_architecture: 'SRPMS', package_format: 'rpm', source_package_formats: ['src.rpm'], signature_format: 'v4', repo: true, }, }, 'osx' => { '10.10' => { architectures: ['x86_64'], package_format: 'dmg', repo: false, }, '10.11' => { architectures: ['x86_64'], package_format: 'dmg', repo: false, }, '10.12' => { architectures: ['x86_64'], package_format: 'dmg', repo: false, }, '10.13' => { architectures: ['x86_64'], package_format: 'dmg', repo: false, }, }, 'redhatfips' => { '7' => { architectures: ['x86_64'], source_architecture: 'SRPMS', package_format: 'rpm', source_package_formats: ['src.rpm'], signature_format: 'v3', repo: true, } }, 'sles' => { '11' => { architectures: ['x86_64', 'i386', 's390x'], source_architecture: 'SRPMS', package_format: 'rpm', source_package_formats: ['src.rpm'], signature_format: 'v3', repo: true, }, '12' => { architectures: ['x86_64', 's390x', 'ppc64le'], source_architecture: 'SRPMS', package_format: 'rpm', source_package_formats: ['src.rpm'], signature_format: 'v4', repo: true, }, '15' => { architectures: ['x86_64', 'ppc64le'], source_architecture: 'SRPMS', package_format: 'rpm', source_package_formats: ['src.rpm'], signature_format: 'v4', repo: true, }, }, 'solaris' => { '10' => { architectures: ['i386', 'sparc'], package_format: 'svr4', repo: false, }, '11' => { architectures: ['i386', 'sparc'], package_format: 'ips', repo: false, }, }, 'ubuntu' => { '14.04' => { codename: 'trusty', architectures: ['amd64', 'i386'], source_architecture: 'source', package_format: 'deb', source_package_formats: DEBIAN_SOURCE_FORMATS, repo: true, }, '16.04' => { codename: 'xenial', architectures: ['amd64', 'i386', 'ppc64el'], source_architecture: 'source', package_format: 'deb', source_package_formats: DEBIAN_SOURCE_FORMATS, repo: true, }, '18.04' => { codename: 'bionic', architectures: ['amd64', 'ppc64el'], source_architecture: 'source', package_format: 'deb', source_package_formats: DEBIAN_SOURCE_FORMATS, repo: true, }, '18.10' => { codename: 'cosmic', architectures: ['amd64', 'ppc64el'], source_architecture: 'source', package_format: 'deb', source_package_formats: DEBIAN_SOURCE_FORMATS, repo: true, }, }, 'windows' => { '2012' => { architectures: ['x86', 'x64'], package_format: 'msi', repo: false, } }, }.freeze
Class Method Summary collapse
-
.all_supported_package_formats ⇒ Array
An Array of Strings, containing all the package formats and source package formats defined in Pkg::Platforms.
-
.arches_for_codename(codename, include_source = false) ⇒ Object
Given a debian codename, return the arches that we build for that codename.
-
.arches_for_platform_version(platform, version, include_source = false) ⇒ Object
Given a platform and version, return the arches that we build for that platform.
-
.by_deb ⇒ Array
Helper meta-method to return all platforms that use .deb packages.
-
.by_package_format(format) ⇒ Array
An Array of Strings, containing all platforms that use
for their packages. -
.by_rpm ⇒ Array
Helper meta-method to return all platforms that use .rpm packages.
-
.codename_for_platform_version(platform, version) ⇒ Object
Given a debian platform and version, return the codename that corresponds to the set.
-
.codename_to_platform_version(codename) ⇒ Object
Given a debian codename, return the platform and version it corresponds to.
-
.codename_to_tags(codename) ⇒ Object
Given a codename, return an array of associated tags.
-
.codenames(platform = 'deb') ⇒ Array
An Array of Strings, containing all of the codenames defined for a given Platform.
-
.formats ⇒ Array
An Array of Strings, containing all of the package formats defined in Pkg::Platforms.
-
.generic_platform_tag(platform) ⇒ Object
Return a supported platform tag for the given platform, not caring about version or architecture.
-
.get_attribute(platform_tag, attribute_name) ⇒ String, Array
The contents of the requested attribute.
- .get_attribute_for_platform_version(platform, version, attribute_name) ⇒ Object
- .package_format_for_tag(platform_tag) ⇒ Object
-
.parse_platform_tag(platform_tag) ⇒ Array
An array of three elements: the platform name, the platform version, and the architecture.
-
.platform_lookup(platform_tag) ⇒ Hash
The hash of data associated with the given platform version.
-
.platform_tags ⇒ Object
Returns an array of all currently valid platform tags.
-
.platform_tags_for_package_format(format) ⇒ Object
Return an array of platform tags associated with a given package format.
- .signature_format_for_platform_version(platform, version) ⇒ Object
- .signature_format_for_tag(platform_tag) ⇒ Object
- .source_architecture_for_platform_tag(platform_tag) ⇒ Object
- .source_package_formats_for_platform_tag(platform_tag) ⇒ Object
-
.supported_platforms ⇒ Array
An array of Strings, containing all of the supported platforms as defined in PLATFORM_INFO.
-
.versions_for_platform(platform) ⇒ Array
An Array of Strings, containing all the supported versions for the given platform.
Class Method Details
.all_supported_package_formats ⇒ Array
Returns An Array of Strings, containing all the package formats and source package formats defined in Pkg::Platforms.
430 431 432 433 434 435 436 437 438 439 |
# File 'lib/packaging/platforms.rb', line 430 def all_supported_package_formats fmts = formats source_fmts = PLATFORM_INFO.flat_map do |_, p| p.collect do |_, r| r[:source_package_formats] end end (fmts + source_fmts).flatten.compact.uniq.to_set.sort end |
.arches_for_codename(codename, include_source = false) ⇒ Object
Given a debian codename, return the arches that we build for that codename
469 470 471 472 |
# File 'lib/packaging/platforms.rb', line 469 def arches_for_codename(codename, include_source = false) platform, version = codename_to_platform_version(codename) arches_for_platform_version(platform, version, include_source) end |
.arches_for_platform_version(platform, version, include_source = false) ⇒ Object
Given a platform and version, return the arches that we build for that platform
486 487 488 489 490 491 492 493 494 495 496 497 498 499 |
# File 'lib/packaging/platforms.rb', line 486 def arches_for_platform_version(platform, version, include_source = false) platform_architectures = get_attribute_for_platform_version(platform, version, :architectures) # get_attribute_for_platform_version will raise an exception if the attribute # isn't found. We don't want this to be a fatal error, we just want to append # the source architecture if it's found source_architecture = [] if include_source begin source_architecture = Array(get_attribute_for_platform_version(platform, version, :source_architecture)) rescue end end return (platform_architectures + source_architecture).flatten end |
.by_deb ⇒ Array
Helper meta-method to return all platforms that use .deb packages
562 563 564 |
# File 'lib/packaging/platforms.rb', line 562 def by_deb by_package_format('deb') end |
.by_package_format(format) ⇒ Array
Returns An Array of Strings, containing all platforms that
use
410 411 412 413 414 415 |
# File 'lib/packaging/platforms.rb', line 410 def by_package_format(format) PLATFORM_INFO.keys.select do |key| formats = PLATFORM_INFO[key].values.collect { |v| v[:package_format] } formats.include? format end end |
.by_rpm ⇒ Array
Helper meta-method to return all platforms that use .rpm packages
571 572 573 |
# File 'lib/packaging/platforms.rb', line 571 def by_rpm by_package_format('rpm') end |
.codename_for_platform_version(platform, version) ⇒ Object
Given a debian platform and version, return the codename that corresponds to the set
464 465 466 |
# File 'lib/packaging/platforms.rb', line 464 def codename_for_platform_version(platform, version) get_attribute_for_platform_version(platform, version, :codename) end |
.codename_to_platform_version(codename) ⇒ Object
Given a debian codename, return the platform and version it corresponds to
453 454 455 456 457 458 459 460 |
# File 'lib/packaging/platforms.rb', line 453 def codename_to_platform_version(codename) PLATFORM_INFO.each do |platform, platform_versions| platform_versions.each do |version, info| return [platform, version] if info[:codename] && codename == info[:codename] end end raise "Unable to find a platform and version for '#{codename}'" end |
.codename_to_tags(codename) ⇒ Object
Given a codename, return an array of associated tags
475 476 477 478 479 480 481 482 |
# File 'lib/packaging/platforms.rb', line 475 def (codename) = [] platform, version = codename_to_platform_version(codename) arches_for_codename(codename).each do |arch| << "#{platform}-#{version}-#{arch}" end end |
.codenames(platform = 'deb') ⇒ Array
Returns An Array of Strings, containing all of the codenames defined for a given Platform.
445 446 447 448 449 450 |
# File 'lib/packaging/platforms.rb', line 445 def codenames(platform = 'deb') releases = by_package_format(platform).flat_map do |p| PLATFORM_INFO[p].values.collect { |r| r[:codename] } end releases.sort end |
.formats ⇒ Array
Returns An Array of Strings, containing all of the package formats defined in Pkg::Platforms.
419 420 421 422 423 424 425 426 |
# File 'lib/packaging/platforms.rb', line 419 def formats fmts = PLATFORM_INFO.flat_map do |_, p| p.collect do |_, r| r[:package_format] end end fmts.to_set.sort end |
.generic_platform_tag(platform) ⇒ Object
Return a supported platform tag for the given platform, not caring about version or architecture
551 552 553 554 555 |
# File 'lib/packaging/platforms.rb', line 551 def generic_platform_tag(platform) version = versions_for_platform(platform).first arch = arches_for_platform_version(platform, version).first return "#{platform}-#{version}-#{arch}" end |
.get_attribute(platform_tag, attribute_name) ⇒ String, Array
Returns the contents of the requested attribute.
394 395 396 397 398 |
# File 'lib/packaging/platforms.rb', line 394 def get_attribute(platform_tag, attribute_name) info = platform_lookup(platform_tag) raise "#{platform_tag} doesn't have information about #{attribute_name} available" unless info.key?(attribute_name) info[attribute_name] end |
.get_attribute_for_platform_version(platform, version, attribute_name) ⇒ Object
400 401 402 403 404 |
# File 'lib/packaging/platforms.rb', line 400 def get_attribute_for_platform_version(platform, version, attribute_name) info = PLATFORM_INFO[platform][version] raise "#{platform_tag} doesn't have information about #{attribute_name} available" unless info.key?(attribute_name) info[attribute_name] end |
.package_format_for_tag(platform_tag) ⇒ Object
514 515 516 |
# File 'lib/packaging/platforms.rb', line 514 def package_format_for_tag(platform_tag) get_attribute(platform_tag, :package_format) end |
.parse_platform_tag(platform_tag) ⇒ Array
Returns An array of three elements: the platform name, the platform version, and the architecture. If the architecture was not included in the original platform tag, then nil is returned in place of the architecture.
345 346 347 348 349 350 351 352 353 354 355 356 357 358 359 360 361 362 363 364 365 366 367 368 369 370 371 372 373 374 375 376 377 378 |
# File 'lib/packaging/platforms.rb', line 345 def parse_platform_tag(platform_tag) platform_elements = platform_tag.split('-') # Look for platform. This is probably the only place where we have to look # for a combination of elements rather than a single element platform = (platform_elements & supported_platforms).first codename = (platform_elements & codenames).first # This is probably a bad assumption, but I'm assuming if we find a codename, # that's more reliable as it's less likely to give us a false match if codename platform, version = codename_to_platform_version(codename) end # There's a possibility that the platform name has a dash in it, in which # case, our assumption that it's an element of the above array is false, # since it would be a combination of elements in that array platform ||= supported_platforms.find { |p| platform_tag =~ /#{p}-/ } version ||= (platform_elements & versions_for_platform(platform)).first # For platform names with a dash in them, because everything is special supported_arches = arches_for_platform_version(platform, version, true) architecture = platform_tag.sub(/^(#{platform}-#{version}|#{codename})-?/, '') fail unless supported_arches.include?(architecture) || architecture.empty? # AIX uses 'ppc' as its architecture in paths and file names architecture = 'ppc' if platform == 'aix' return [platform, version, architecture] rescue raise "Could not verify that '#{platform_tag}' is a valid tag" end |
.platform_lookup(platform_tag) ⇒ Hash
Returns The hash of data associated with the given platform version.
384 385 386 387 |
# File 'lib/packaging/platforms.rb', line 384 def platform_lookup(platform_tag) platform, version, _ = parse_platform_tag(platform_tag) PLATFORM_INFO[platform][version] end |
.platform_tags ⇒ Object
Returns an array of all currently valid platform tags
502 503 504 505 506 507 508 509 510 511 512 |
# File 'lib/packaging/platforms.rb', line 502 def = [] PLATFORM_INFO.each do |platform, platform_versions| platform_versions.each do |version, info| info[:architectures].each do |arch| << "#{platform}-#{version}-#{arch}" end end end end |
.platform_tags_for_package_format(format) ⇒ Object
Return an array of platform tags associated with a given package format
535 536 537 538 539 540 541 542 543 544 545 546 547 |
# File 'lib/packaging/platforms.rb', line 535 def (format) = [] PLATFORM_INFO.each do |platform, platform_versions| platform_versions.each do |version, info| info[:architectures].each do |architecture| if info[:package_format] == format << "#{platform}-#{version}-#{architecture}" end end end end end |
.signature_format_for_platform_version(platform, version) ⇒ Object
522 523 524 |
# File 'lib/packaging/platforms.rb', line 522 def signature_format_for_platform_version(platform, version) get_attribute_for_platform_version(platform, version, :signature_format) end |
.signature_format_for_tag(platform_tag) ⇒ Object
518 519 520 |
# File 'lib/packaging/platforms.rb', line 518 def signature_format_for_tag(platform_tag) get_attribute(platform_tag, :signature_format) end |
.source_architecture_for_platform_tag(platform_tag) ⇒ Object
526 527 528 |
# File 'lib/packaging/platforms.rb', line 526 def source_architecture_for_platform_tag(platform_tag) get_attribute(platform_tag, :source_architecture) end |
.source_package_formats_for_platform_tag(platform_tag) ⇒ Object
530 531 532 |
# File 'lib/packaging/platforms.rb', line 530 def source_package_formats_for_platform_tag(platform_tag) get_attribute(platform_tag, :source_package_formats) end |
.supported_platforms ⇒ Array
Returns An array of Strings, containing all of the supported platforms as defined in PLATFORM_INFO.
326 327 328 |
# File 'lib/packaging/platforms.rb', line 326 def supported_platforms PLATFORM_INFO.keys end |
.versions_for_platform(platform) ⇒ Array
Returns An Array of Strings, containing all the supported versions for the given platform.
332 333 334 335 336 |
# File 'lib/packaging/platforms.rb', line 332 def versions_for_platform(platform) PLATFORM_INFO[platform].keys rescue raise "No information found for '#{platform}'" end |